/*
 *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
 *	This will be free software, but only when it is finished.
 */

/* Information needed by the database search routines */

#include "splay.h"

extern struct sptree *spt_files, *spt_modcheck;

#define	DESC_FILEP	0	/* FILE * */
#define	DESC_NDBMP	1	/* DB * */

struct search_info {
	char		*file;
	unsigned char	*key;
	time_t		ttl;
	char		*subtype;
};

#ifndef	MALLOC_TRACE
extern unsigned int symbol();
#endif	/* !MALLOC_TRACE */
