s~^ZCONFIG=.*~ZCONFIG=	/etc/zmailer.conf~
s~^HOSTENV=.*~HOSTENV=	SunOS5~
s~^DEFS=.*~DEFS=~
s~^SHELL=.*~SHELL=		/bin/sh~
s~^MAILBIN=.*~MAILBIN=	/slocal/lib/mail~
s~^MAILSHARE=.*~MAILSHARE=	/slocal/share/mail~
s~^MAILVAR=.*~MAILVAR=	/var/db/mail~
s~^MAILBOX=.*~MAILBOX=	/var/mail~
s~^POSTOFFICE=.*~POSTOFFICE=	/var/spool/postoffice~
s~^LOGDIR=.*~LOGDIR=		/var/log~
s~^#LOGLEVEL=.*~#LOGLEVEL=	file: recipient:~
s~^MANDIR=.*~MANDIR=		/local/man~
s~^CC=.*~CC=		gcc # gcc -Wall -pedantic~
s~^#COPTS=.*~#COPTS=		-traditional -g -O~
s~^COPTS=.*~COPTS=		-g -O~
s~^RANLIB=.*~RANLIB=		true # : ar does the work of ranlib under System V~
s~^INSTALL=.*~INSTALL=	/usr/ucb/install -o root~
s~^NOBODY=.*~NOBODY=29999~
s~^#NROUTERS=.*~#NROUTERS=10~
s~^#MAILSERVER=.*~#MAILSERVER=	yonge.cs.toronto.edu~
s~^SMTPOPTIONS=.*~SMTPOPTIONS=	-l /var/log/smtpserver~
s~^PUNTHOST=.*~PUNTHOST=	mail.cs.toronto.edu~
s~^SMARTHOST=.*~SMARTHOST=	mail.cs.toronto.edu~
s~^ZSH_LIB=.*~ZSH_LIB=	~
s~^SMTPSERVER_LIB=.*~SMTPSERVER_LIB=	 -lresolv -lsocket -lnsl~
s~^HOLD_LIB=.*~HOLD_LIB=	 -lresolv -lsocket -lnsl~
s~^LIBDB_LIB=.*~LIBDB_LIB=	 -lresolv -lsocket -lnsl~
s~^SCHEDULER_INCL=.*~SCHEDULER_INCL=	~
s~^MAILQ_INCL=.*~MAILQ_INCL=	~
s~^RMAIL_INCL=.*~RMAIL_INCL=	~
s~^SMTP_INCL=.*~SMTP_INCL=	~
s~^MAILBOX_INCL=.*~MAILBOX_INCL=	~
s~^LIBZ_LIB=.*~LIBZ_LIB=	~
s~^SM_LIB=.*~SM_LIB=	 -lresolv -lsocket -lnsl~
s~^ROUTER_INCL=.*~ROUTER_INCL=	~
s~^LIBC_INCL=.*~LIBC_INCL=	~
s~^ERRORMAIL_INCL=.*~ERRORMAIL_INCL=	~
s~^SENDMAIL_INCL=.*~SENDMAIL_INCL=	~
s~^SCHEDULER_LIB=.*~SCHEDULER_LIB=	 -lresolv -lsocket -lnsl~
s~^MAILQ_LIB=.*~MAILQ_LIB=	 -lresolv -lsocket -lnsl~
s~^RMAIL_LIB=.*~RMAIL_LIB=	 -lresolv -lsocket -lnsl~
s~^ZSH_INCL=.*~ZSH_INCL=	~
s~^SMTP_LIB=.*~SMTP_LIB=	 -lresolv -lsocket -lnsl~
s~^SMTPSERVER_INCL=.*~SMTPSERVER_INCL=	~
s~^MAILBOX_LIB=.*~MAILBOX_LIB=	 -lmail -lresolv -lsocket -lnsl~
s~^HOLD_INCL=.*~HOLD_INCL=	~
s~^LIBDB_INCL=.*~LIBDB_INCL=	~
s~^ROUTER_LIB=.*~ROUTER_LIB=	 -lresolv -lsocket -lnsl~
s~^LIBC_LIB=.*~LIBC_LIB=	~
s~^LIBZ_INCL=.*~LIBZ_INCL=	~
s~^ERRORMAIL_LIB=.*~ERRORMAIL_LIB=	 -lresolv -lsocket -lnsl~
s~^SM_INCL=.*~SM_INCL=	~
s~^SENDMAIL_LIB=.*~SENDMAIL_LIB=	 -lresolv -lsocket -lnsl~
